# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['slk', 'slk.chain', 'slk.classes', 'slk.config', 'slk.repl', 'slk.utils']

package_data = \
{'': ['*'], 'slk.config': ['templates/*']}

install_requires = \
['Jinja2>=3.0.3,<4.0.0',
 'pytest>=6.2.5,<7.0.0',
 'python-dotenv>=0.19.1,<0.20.0',
 'tabulate>=0.8.9,<0.9.0',
 'websockets>=9.1,<10.0',
 'xrpl-py>=1.3.0,<2.0.0']

entry_points = \
{'console_scripts': ['sidechain-config = slk.config:create_config_files',
                     'sidechain-shell = slk.run_shell:main']}

setup_kwargs = {
    'name': 'sidechain-launch-kit',
    'version': '1.0.0b1',
    'description': 'Python scripts to tests and explore side chains',
    'long_description': '# Sidechain Launch Kit\n\n## Introduction\n\nThis directory contains python scripts to tests and explore side chains.\n\nThis document walks through the steps to setup a side chain running on your local machine and make your first cross chain transfers.\n\n## Get Ready\n\nThis section describes how to install the python dependencies, create the environment variables, and create the configuration files that scripts need to run correctly.\n\n### Install the sidechain launch kit\n\n```\npip install sidechain-launch-kit\n```\n\n### Build rippled\n\nCheckout the `sidechain` branch from the [rippled repository](https://github.com/ripple/rippled/tree/sidechain), and follow the usual process to build rippled.\n\n### Environment variables\n\nThe python scripts need to know the locations of two files and one directory. These can be specified through command line arguments, by adding variables in the `.env` file, or by setting environment variables.\n\n1. The location of the rippled executable used for main chain servers. Either set the environment variable `RIPPLED_MAINCHAIN_EXE` (in your system or in the `.env` file) or use the command line switch `--exe_mainchain`. Until a new RPC is integrated into the main branch (this will happen very soon), use the code built from the sidechain branch as the main chain exe.\n2. The location of the rippled executable used for side chain servers. Either set the environment variable `RIPPLED_SIDECHAIN_EXE` (in your system or in the `.env` file) or use the command line switch `--exe_sidechain`. This should be the rippled executable built from the sidechain branch.\n3. The location of the directory that has the rippled configuration files. Either set the environment variable `RIPPLED_SIDECHAIN_CFG_DIR` (in your system or in the `.env` file) or use the command line switch `--cfgs_dir`. The configuration files do not exist yet. There is a script to create these for you. For now, just choose a location  where the files should live and make sure that directory exists.\n4. The number of federators to have in the sidechain. This must be a number between 1 and 8. Either set the environment variable `NUM_FEDERATORS` (in your system or in the `.env` file) or use the command line switch `--num_federators`. The script that creates the config files uses this number to create the right config files for that number of federators.\n\n#### Additional environment variables for connecting to mainnet/devnet/testnet\n\n5. The IP address of a node on the mainchain. Either set the environment variable `MAINNET` (in your system or in the `.env` file) or use the command line switch `--mainnet`. This is used in both the script that creates the config files and in the script that runs the sidechain.\n6. The public Websocket port of the mainchain node. Either set the environment variable `MAINNET_PORT` (in your system or in the `.env` file) or use the command line switch `--mainnet_port`. This is used in both the script that creates the config files and in the script that runs the sidechain.\n7. The seed of the issuer of the cross-chain token. Either set the environment variable `IOU_ISSUER` (in your system or in the `.env` file) or use the command line switch `--iou_issuer`. This is used in both the script that creates the config files and in the script that runs the sidechain.\n8. The seed of the door account on the mainchain. Either set the environment variable `DOOR_ACCOUNT_SEED` (in your system or in the `.env` file) or use the command line switch `--door_seed`. This is used in the script that creates the config files.\n\nSetting environment variables can be very convenient. For example, a small script can be sourced to set these environment variables when working with side chains.\n\n\n### Creating configuration files\n\nAssuming rippled is built, the three environment variables are set, and the python environment is activated, run the following script:\n```\nsidechain-config --assets usd\n```\n\nThere should now be many configuration files in the directory specified by the `RIPPLED_SIDECHAIN_CFG_DIR` environment variable. The `--assets usd` creates a sample cross chain asset for USD -> USD transfers.\n\n## Running the interactive shell\n\nThere is an interactive shell that can be used to explore side chains. It will use the configuration files built above to spin up test rippled main chain running in standalone mode as well as 5 side chain federators running in regular consensus mode.\n\nTo start the shell, run the following script:\n```\nsidechain-shell\n```\n\nThe shell will not start until the servers have synced. It may take a minute or two until they do sync. The script should give feedback while it is syncing.\n\nOnce the shell has started, the following message should appear:\n```\nWelcome to the sidechain test shell.   Type help or ? to list commands.\n\nSSX>\n```\n\nType the command `server_info` to make sure the servers are running. An example output would be:\n```\nSSX> server_info\n           pid                                  config  running server_state  ledger_seq complete_ledgers\nmain 0  136206                   mainchain/rippled.cfg     True    proposing          75             2-75\nside 0  136230                 sidechain_0/rippled.cfg     True    proposing          92             1-92\n     1  136231                 sidechain_1/rippled.cfg     True    proposing          92             1-92\n     2  136232                 sidechain_2/rippled.cfg     True    proposing          92             1-92\n     3  136233                 sidechain_3/rippled.cfg     True    proposing          92             1-92\n     4  136234                 sidechain_4/rippled.cfg     True    proposing          92             1-92\n```\n\nOf course, you\'ll see slightly different output on your machine. The important thing to notice is there are two categories, one called `main` for the main chain and one called `side` for the side chain. There should be a single server for the main chain and five servers for the side chain.\n\nNext, type the `balance` command, to see the balances of the accounts in the address book:\n```\nSSX> balance\n                           balance currency peer limit\n     account\nmain root    99,999,989,999.999985      XRP\n     door             9,999.999940      XRP\nside door    99,999,999,999.999954      XRP\n```\n\nThere are two accounts on the main chain: `root` and `door`; and one account on the side chain: `door`. These are not user accounts. Let\'s add two user accounts, one on the main chain called `alice` and one on the side chain called `bob`. The `new_account` command does this for us.\n\n```\nSSX> new_account mainchain alice\nSSX> new_account sidechain bob\n```\n\nThis just added the accounts to the address book, but they don\'t exist on the ledger yet. To do that, we need to fund the accounts with a payment. For now, let\'s just fund the `alice` account and check the balances. The `pay` command makes a payment on one of the chains:\n\n```\nSSX> pay mainchain root alice 5000\nSSX> balance\n                           balance currency peer limit\n     account\nmain root    99,999,984,999.999969      XRP\n     door             9,999.999940      XRP\n     alice            5,000.000000      XRP\nside door    99,999,999,999.999954      XRP\n     bob                  0.000000      XRP\n```\n\nFinally, let\'s do something specific to side chains: make a cross chain payment. The `xchain` command makes a payment between chains:\n\n```\nSSX> xchain mainchain alice bob 4000\nSSX> balance\n                           balance currency peer limit\n     account\nmain root    99,999,984,999.999969      XRP\n     door            13,999.999940      XRP\n     alice              999.999990      XRP\nside door    99,999,995,999.999863      XRP\n     bob              4,000.000000      XRP\n```\n\nNote: the account reserve on the side chain is 100 XRP. The cross chain amount must be greater than 100 XRP or the payment will fail.\n\nMaking a cross chain transaction from the side chain to the main chain is similar:\n```\nSSX> xchain sidechain bob alice 2000\nSSX> balance\n                           balance currency peer limit\n     account\nmain root    99,999,984,999.999969      XRP\n     door            11,999.999840      XRP\n     alice            2,999.999990      XRP\nside door    99,999,997,999.999863      XRP\n     bob              1,999.999990      XRP\n```\n\nIf you typed `balance` very quickly, you may catch a cross chain payment in progress and the XRP may be deducted from bob\'s account before it is added to alice\'s. If this happens just wait a couple seconds and retry the command. Also note that accounts pay a ten drop fee when submitting transactions.\n\nFinally, exit the program with the `quit` command:\n```\nSSX> quit\nThank you for using the sidechain shell. Goodbye.\n\n\nWARNING: Server 0 is being stopped. RPC commands cannot be sent until this is restarted.\n```\n\nIgnore the warning about the server being stopped.\n\n## Conclusion\n\nThose two cross chain payments are a "hello world" for side chains. It makes sure you\'re environment is set up correctly.\n',
    'author': 'Mayukha Vadari',
    'author_email': 'mvadari@ripple.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/xpring-eng/sidechain-launch-kit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.10',
}


setup(**setup_kwargs)
